//---------------------------------------------------------------------------


#pragma hdrstop

#include "Signal.h"
#include <stdlib.h>
#include <math.h>

//---------------------------------------------------------------------------

#pragma package(smart_init)

Signal::Signal()
{
	shape = WAVE_SINUS;
	fAmplitude = 1.0;
	fOffset = 0.0f;
}

short* Signal::Generate(unsigned int count, float fDivY)
{
	if (count > sizeof(data)/sizeof(data[0]))
		return NULL;

	for (unsigned int i=0; i!=count; ++i)
	{
		fData[i] = 0;
	}

		
	switch (shape)
	{
	//case WAVE_DC:
	//	for (unsigned int i=0; i!=count; ++i)
	//	{
	//		fData[i] += fAmplitude/fDivY;
	//	}
	//	break;
	case WAVE_SINUS:
		for (unsigned int i=0; i!=count; ++i)
		{
			fData[i] += fAmplitude/fDivY * sin(2.0 * M_PI * ((float)i/(float)count));
		}
		break;
	case WAVE_SQUARE:
		for (unsigned int i=0; i!=count; ++i)
		{
			if ( i<count/2 )
				fData[i] += fAmplitude/fDivY;
			else
				fData[i] += (-1.0) * fAmplitude/fDivY;
		}
		break;
	case WAVE_NOISE:
		for (unsigned int i=0; i!=count; ++i)
		{
			fData[i] += fAmplitude/fDivY * (((float)random(100) - 50.0)/50.0);
		}
		break;
	case WAVE_TRIANGLE: {
		float delta = fAmplitude * 4 / count;
		float val = -1.0f * fAmplitude;
		unsigned int i;
		for (i=0; i<count/2; ++i)
		{
			fData[i] = val/fDivY;
			val += delta;
		}
		for (i; i<count; ++i)
		{
			fData[i] = val/fDivY;
			val -= delta;
		}
		break;
	}
	case WAVE_SAWTOOTH: {
		float delta = fAmplitude * 2 / count;
		float val = -1.0f * fAmplitude;
		unsigned int i;
		for (i=0; i<count; ++i)
		{
			fData[i] = val/fDivY;
			val += delta;
		}
		break;
	}
	case WAVE_SAWTOOTH_REV: {
		float delta = fAmplitude * 2 / count;
		float val = 1.0f * fAmplitude;
		unsigned int i;
		for (i=0; i<count; ++i)
		{
			fData[i] = val/fDivY;
			val -= delta;
		}
		break;
	}
	default:
		break;
	}

             
	for (unsigned int i=0; i!=count; ++i)
	{
		data[i] = fData[i] + fOffset/fDivY;
	}
	return data;
}


int Signal::GetTypeDescription(int type, std::string &desc)
{
	switch(type) {
//	case	WAVE_NONE:
//		desc = "none";
//		break;
//	case	WAVE_DC:
//		desc = "DC offset";
//		break;
	case	WAVE_SINUS:
		desc = "Sinus";
		break;
	case	WAVE_SQUARE:
		desc = "Square";
		break;
	case	WAVE_NOISE:
		desc = "Noise";
		break;
	case	WAVE_TRIANGLE:
		desc = "Triangle";
		break;
	case	WAVE_SAWTOOTH:
		desc = "Sawtooth";
		break;
	case	WAVE_SAWTOOTH_REV:
		desc = "Sawtooth reversed";
		break;
	default:
		desc = "";
		return -1;
	}
	return 0;
}

